#include <Windows.h>
#include <stdio.h>
#include "Ft602.h"
#include "getopt.h"

#ifdef _WIN64
#pragma comment(lib, "x64/ft602.lib")
#else
#pragma comment(lib, "Win32/ft602.lib")
#endif

HANDLE hNotification;


static VOID NotificationCallback(PVOID pvCallbackContext, E_FT_NOTIFICATION_CALLBACK_TYPE eCallbackType, PVOID pvCallbackInfo)
{
	/* signal the event */
	SetEvent(hNotification);
	printf("Notificatin callback received\n");
}



void display_usage(char* prog)
{
	printf("\nusage:\n    %s -a i2cAddr -o offset <options> \n", prog);
	printf("    %s -i \n",prog);

	printf("       : <options> \n");
	printf("       :         -r (for read) \n");
	printf("       :         -w (for write) \n");
	printf("       :         -l (size to read/write) \n");
	printf("       :         -i (notification on interrupt (IRQ line)) \n");

}

static struct option long_options[] = {
	{ "addr", required_argument, 0, 'a' },
	{ "offset", required_argument, 0, 'o' },
	{ "read", no_argument, 0, 'r' },
	{ "write", no_argument, 0, 'w' },
	{ "help", no_argument, 0, 'h' },
	{ "length", required_argument, 0, 'l' },
	{ "irq", no_argument, 0, 'i'},
	{ 0, 0, 0, 0 }
};


int main(int argc, char* argv[])
{
	FT_HANDLE ftHandle;
	FT_STATUS ftStatus;
	bool bWrite = FALSE, bIrq = FALSE;
	int i2caddr = 0, val = 0, len = 2;
	int reg = 0;
	int rdMem[4];	
	int long_index = 0;
	int opt;

	if(argc < 2)
	{
		display_usage(argv[0]);
		return 0;
	}

	while ((opt = getopt_long(argc, argv, "hrwia:o:l:",
		long_options, &long_index)) != -1) {

		switch (opt)
		{
			default:
			case 'h':
				display_usage(argv[0]);
				return 0;
			case 'l' : len = strtoul(optarg, NULL, 0);
				break;
			case 'a': i2caddr = strtoul(optarg, NULL, 0);
				break;
			case 'o': reg = strtoul(optarg, NULL, 0);
				break;
			case 'r': bWrite = FALSE; 
				break;
			case 'w': bWrite = TRUE;
				break;
			case 'i': bIrq = TRUE;
				break;
			
		}
	}
			
			
	ftStatus = FT_Create(0, FT_OPEN_BY_INDEX, &ftHandle);
	if (FT_FAILED(ftStatus))
	{
		printf("FT_Create failed %d\n", ftStatus);
		goto exit;
	}

	if (bIrq) /* IRQ notification*/
	{
		/* global event */
		hNotification = CreateEvent(NULL, TRUE, FALSE, NULL);
		if (!hNotification)
		{
			goto exit;
		}
		
		/* set a callback function */
		FT_SetNotificationCallback(ftHandle, NotificationCallback, NULL);
		
		/* wait for the event to signal*/
		DWORD dwRet = WaitForSingleObject(hNotification, INFINITE);
		
	}
	else
	{
		struct i2c_access i2c;

		/* initialize the i2c strutcure */
		i2c.addr = i2caddr;
		i2c.len = len;
		i2c.read_access = !bWrite;
		i2c.reg = reg;
		i2c.reg_is_16_bit = 0;


		FT_I2CAccess(ftHandle, &i2c, (uint8_t*)&rdMem[0]);
		if (!bWrite)
			printf("value returned: 0x%x \n", rdMem[0]);

	}
	FT_Close(ftHandle);

exit:
	return 0;
}